@@ -80,7 +80,9 @@ RenumberEdges <- function (parent, child, ...) {
   list(oenn[, 1], oenn[, 2])
 }
 
-#' Reorder trees
+#' Reorder tree edges and nodes
+#'
+#' Functions for systematically ordering the internal edges of trees.
 #'
 #' `Reorder()` is a wrapper for \code{ape:::.reorder_ape}.
 #' Calling this C function directly is approximately twice as fast as using
@@ -118,8 +120,9 @@ RenumberEdges <- function (parent, child, ...) {
 #' as `Preorder()` and are intended for use where performance is at a premium.
 #'
 #'
-#' `Postorder()` is modified from the 'ape' function to return a specific
-#' order: edges are listed from the node that subtends the smallest
+#' `Postorder()` is an optimized implementation that if `sizeSort = TRUE`
+#' return a specific order:
+#' edges are listed from the node that subtends the smallest
 #' subtree to the one that subtends the largest (i.e. the root node), with
 #' all of a node's descendant edges listed adjacently.  If a tree is already
 #' in postorder, it will not be rearranged unless `force = TRUE`.
@@ -129,8 +132,9 @@ RenumberEdges <- function (parent, child, ...) {
 #' crashing R.
 #'
 #' Trees with >8191 leaves require additional memory and are not handled
-#' at present.  If you need to process such large trees, please contact the
-#' maintainer for advice.
+#' by `Postorder()` at present.
+#' If you need to process such large trees, please contact the maintainer for
+#' advice.
 #'
 #' @template treeParam
 #' @template nTipParam
@@ -259,19 +263,22 @@ ApePostorder.multiPhylo <- function (tree, nTip, edge) {
 #' @param force Logical specifying whether to rearrange trees already in
 #' postorder, in order to ensure edges are ordered in the 'TreeTools' fashion.
 #' @export
-Postorder <- function (tree, force = FALSE, renumber = FALSE) {
+Postorder <- function (tree, force = FALSE, renumber = FALSE,
+                       sizeSort = TRUE) {
   UseMethod('Postorder')
 }
 
 #' @rdname Reorder
 #' @export
-Postorder.phylo <- function (tree, force = FALSE, renumber = FALSE) {
+Postorder.phylo <- function (tree, force = FALSE, renumber = FALSE,
+                             sizeSort = TRUE) {
   if (is.null(attr(tree, "order"))
       || attr(tree, "order") != "postorder"
       || (force &&
           (is.null(attr(tree, 'suborder')) ||
            attr(tree, 'suborder') != 'TreeTools'))) {
-    tree$edge <- Postorder(tree$edge, renumber = renumber)
+    tree$edge <- Postorder(tree$edge, renumber = renumber,
+                           sizeSort = sizeSort)
     tree$edge.length <- NULL
     attr(tree, "order") <- "postorder"
     attr(tree, "suborder") <- "TreeTools"
@@ -281,19 +288,24 @@ Postorder.phylo <- function (tree, force = FALSE, renumber = FALSE) {
 
 #' @rdname Reorder
 #' @export
-Postorder.NULL <- function (tree, force = FALSE, renumber = FALSE) NULL
+Postorder.NULL <- function (tree, force = FALSE, renumber = FALSE,
+                            sizeSort = TRUE) NULL
 
 #' @rdname Reorder
 #' @export
-Postorder.list <- function (tree, force = FALSE, renumber = FALSE) {
-  lapply(tree, Postorder, force = force, renumber = renumber)
+Postorder.list <- function (tree, force = FALSE, renumber = FALSE,
+                            sizeSort = TRUE) {
+  lapply(tree, Postorder, force = force, renumber = renumber,
+         sizeSort = sizeSort)
 }
 
 
 #' @rdname Reorder
 #' @export
-Postorder.multiPhylo <- function (tree, force = FALSE, renumber = FALSE) {
-  tree[] <- lapply(tree, Postorder, force = force, renumber = renumber)
+Postorder.multiPhylo <- function (tree, force = FALSE, renumber = FALSE,
+                                  sizeSort = TRUE) {
+  tree[] <- lapply(tree, Postorder, force = force, renumber = renumber,
+                   sizeSort = sizeSort)
   attr(tree, 'order') <- 'postorder'
   tree
 }
@@ -303,8 +315,9 @@ Postorder.multiPhylo <- function (tree, force = FALSE, renumber = FALSE) {
 #' `edge` entry of a tree of class `phylo`, and returns a two-column array
 #' corresponding to `tree`, with edges listed in postorder
 #' @export
-Postorder.numeric <- function (tree, force = FALSE, renumber = FALSE) {
-  ret <- postorder_edges(tree - 1L)
+Postorder.numeric <- function (tree, force = FALSE, renumber = FALSE,
+                               sizeSort = TRUE) {
+  ret <- postorder_edges(tree - 1L, sizeSort[1])
   if (renumber) {
     internals <- unique(tree[, 1])
     nTip <- min(internals) - 1L
@@ -447,7 +460,14 @@ RenumberTips.phylo <- function (tree, tipOrder) {
   newOrder <- TipLabels(tipOrder, single = TRUE)
   if (identical(startOrder, newOrder)) return (tree)
   if (length(startOrder) != length(newOrder)) {
-    stop("Tree labels and tipOrder must match")
+    startOnly <- setdiff(startOrder, newOrder)
+    newOnly <- setdiff(newOrder, startOrder)
+    stop("Tree labels and tipOrder must match.",
+         if (length(newOnly)) "\n  Missing in `tree`: ",
+         paste0(newOnly, collapse = ', '),
+         if (length(startOnly)) "\n  Missing in `tipOrder`: ",
+         paste0(startOnly, collapse = ', ')
+         )
   }
 
   nTip <- length(startOrder)
@@ -455,7 +475,10 @@ RenumberTips.phylo <- function (tree, tipOrder) {
   tips <- child <= nTip
 
   matchOrder <- match(startOrder, newOrder)
-  if (any(is.na(matchOrder))) stop("All tree labels must occur in tipOrder")
+  if (any(is.na(matchOrder))) {
+    stop("Tree labels ", paste0(startOrder[is.na(matchOrder)], collapse = ', '),
+         " missing from `tipOrder`")
+  }
   tree$edge[tips, 2] <- matchOrder[tree$edge[tips, 2]]
   tree$tip.label <- newOrder
   tree
